




import xlwings as xw
import gspread
from oauth2client.service_account import ServiceAccountCredentials
import schedule
import time
from datetime import datetime

def convert_to_serializable(value):
    if isinstance(value, datetime):
        return value.isoformat()
    return value

def update_google_sheet():
    # Open the Excel workbook
    workbook = xw.Book('ExcelFile.xlsm')

    # Setup Google Sheets API credentials
    scope = ['https://spreadsheets.google.com/feeds', 'https://www.googleapis.com/auth/drive']
    creds = ServiceAccountCredentials.from_json_keyfile_name('Key.json', scope)
    client = gspread.authorize(creds)
    google_sheets_file = client.open('Data from excel')

    # Loop through each worksheet in the Excel workbook
    for worksheet in workbook.sheets:
        sheet_name = worksheet.name
        
        try:
            # Try to open the Google Sheet tab with the same name as the Excel sheet
            sheet = google_sheets_file.worksheet(sheet_name)
        except gspread.exceptions.WorksheetNotFound:
            # If it doesn't exist, create it
            sheet = google_sheets_file.add_worksheet(title=sheet_name, rows="1000", cols="26")

        # Prepare the data from the current worksheet
        data = []
        used_range = worksheet.used_range
        
        for row in used_range.rows:
            if row.value is not None:  # Skip empty rows
                converted_row = [convert_to_serializable(cell) for cell in row.value]
                data.append(converted_row)

        # Clear existing data in the Google Sheet tab and update with new data
        sheet.clear()
        if data:  # Only update if there's data to write
            sheet.update(range_name='A1', values=data)

    print(f"Google Sheet update complete at {datetime.now().strftime('%Y-%m-%d %H:%M:%S')}")




# First, run the update_google_sheet function immediately
update_google_sheet()

# Then schedule it to run every 1 minute
schedule.every(1).minutes.do(update_google_sheet)

# Main loop
while True:
    schedule.run_pending()
    time.sleep(1)



